/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.auth;

import static java.util.Locale.ROOT;
import javax.annotation.*;
import javax.xml.bind.annotation.XmlElement;

import net.java.truelicense.core.util.*;

/**
 * A model for storing authenticated objects (artifacts).
 * This class is used by {@linkplain Repository repositories} to store
 * artifacts for {@linkplain Repository#sign signing} and
 * {@linkplain Repository#verify verifying}.
 *
 * @author Christian Schlichtherle
 */
@Nullable
public class RepositoryModel {

    private String artifact, signature, algorithm;

    @XmlElement(required = true)
    public final String getArtifact() { return artifact; }

    public final void setArtifact(final String body) {
        this.artifact = body;
    }

    @XmlElement(required = true)
    public final String getSignature() { return signature; }

    /**
     * Sets the signature body.
     * The assumed signature type is {@code application/octet-stream} and its
     * transfer encoding is {@code base64}.
     *
     * @param body the signature body.
     */
    public void setSignature(final String body) {
        this.signature = body;
    }

    @XmlElement(required = true)
    public final String getAlgorithm()  { return algorithm; }

    public final void setAlgorithm(final String algorithm) {
        this.algorithm = algorithm;
    }

    @Override
    @SuppressWarnings("AccessingNonPublicFieldOfAnotherObject")
    public boolean equals(final Object obj) {
        if (this == obj) return true;
        if (!(obj instanceof RepositoryModel)) return false;
        final RepositoryModel that = (RepositoryModel) obj;
        return  Objects.equals(this.signature, that.signature) &&
                Strings.equalsIgnoreCase(this.algorithm, that.algorithm);
    }

    @Override
    public int hashCode() {
        int c = 17;
        c = 31 * c + Objects.hashCode(signature);
        c = 31 * c + Objects.hashCode(Strings.toLowerCase(algorithm, ROOT));
        return c;
    }
}
