/*
 * Copyright (C) 2005-2013 Schlichtherle IT Services.
 * All rights reserved. Use is subject to license terms.
 */
package net.java.truelicense.core.util;

import java.util.Locale;
import javax.annotation.concurrent.Immutable;
import static net.java.truelicense.core.util.Objects.*;

/**
 * A formatted internationalized message based on
 * {@link FormattedResourceBundle}.
 *
 * @author Christian Schlichtherle
 */
@Immutable
public abstract class FormattedMessage implements Message {

    private final String key;
    private final Object[] args;

    /**
     * Constructs an abstract message.
     *
     * @param key the key to lookup in the resource bundle.
     * @param args the arguments for formatting the looked up message.
     */
    protected FormattedMessage(
            final String key,
            final Object... args) {
        this.key = requireNonNull(key);
        this.args = args.clone();
    }

    /** Returns the base name of the resource bundle to use. */
    protected abstract String baseName();

    @Override
    public final String toString() {
        return toString(Locale.getDefault());
    }

    @Override public String toString(Locale locale) {
        return 0 == args.length
                ? bundle(locale).lookup(key)
                : bundle(locale).format(key, args);
    }

    private FormattedResourceBundle bundle(Locale locale) {
        return FormattedResourceBundle.bundle(baseName(), locale);
    }
}
